"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.imageToWebpPlugin = imageToWebpPlugin;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const sharp_1 = __importDefault(require("sharp"));
const pluginName = 'vite-plugin-image-to-webp';
function getBeautifiedFileSize(size) {
    const sizeInBytes = size;
    let readableSize = '';
    if (Math.abs(sizeInBytes) < 1024) {
        readableSize = `${sizeInBytes} bytes`;
    }
    else {
        const sizeInKB = Math.round(sizeInBytes / 1024);
        readableSize = `${sizeInKB} kB`;
    }
    return readableSize;
}
function logBeautifiedConversionData(label, conversionData, options) {
    let beforeNice = getBeautifiedFileSize(conversionData.before);
    let afterNice = getBeautifiedFileSize(conversionData.after);
    let totalPercentageSaved = Math.round((1 - conversionData.after / conversionData.before) * 100);
    let difference = getBeautifiedFileSize(conversionData.before - conversionData.after);
    if (totalPercentageSaved < 0) {
        console.log(`${label}\x1b[35m${beforeNice} => ${afterNice} - \x1b[31m${totalPercentageSaved}% MORE (${difference})`);
        console.log('Maybe try to change the options. Your current vite.config.ts options:', options.webpQuality);
    }
    else {
        console.log(`${label}\x1b[35m${beforeNice} => ${afterNice} - \x1b[36m${totalPercentageSaved}% less (${difference})`);
    }
}
function imageToWebpPlugin(options) {
    options = Object.assign({ imageFormats: ['jpg', 'jpeg', 'png'], webpQuality: {}, destinationFolder: 'dist' }, options);
    return {
        name: pluginName,
        apply: 'build',
        async closeBundle() {
            let imageFilesRegex = new RegExp(`\.(${options.imageFormats.join('|')})$`);
            const allDistFiles = [];
            const walk = (dir) => {
                fs_1.default.readdirSync(dir).forEach(file => {
                    const filePath = path_1.default.join(dir, file);
                    if (fs_1.default.statSync(filePath).isDirectory()) {
                        walk(filePath);
                    }
                    allDistFiles.push(filePath);
                });
            };
            walk(options.destinationFolder);
            let allImageFilesToBeReplaced = [];
            let imageConversionData = [];
            try {
                for (const originalFilePath of allDistFiles) {
                    // if it's the one of the searched image files
                    if (imageFilesRegex.test(originalFilePath)) {
                        allImageFilesToBeReplaced.push(originalFilePath);
                        const destinationWebpPath = originalFilePath.replace(imageFilesRegex, '.webp');
                        // convert image to .webp
                        await (0, sharp_1.default)(originalFilePath)
                            .webp(options.webpQuality)
                            .toFile(destinationWebpPath);
                        // aggregate the stats for the new and old image
                        // in order to calculate the compression success (kB and %)
                        const originalFileStats = fs_1.default.statSync(originalFilePath);
                        const newFileStats = fs_1.default.statSync(destinationWebpPath);
                        imageConversionData.push({
                            before: originalFileStats.size,
                            after: newFileStats.size,
                            percentage: Math.round((1 - newFileStats.size / originalFileStats.size) * 100)
                        });
                        // remove the original file (in the dist folder only)
                        fs_1.default.unlinkSync(originalFilePath);
                    }
                }
                let notLoggedYet = true;
                // intro
                console.log(`\n\x1b[36m${pluginName}: \x1b[0m`);
                let i = 0;
                // for all js, html and css files:
                const replacePaths = (filePath) => {
                    let fileContent = fs_1.default.readFileSync(filePath, 'utf-8');
                    // TODO: make it more efficient. no need yet but there has to be a way
                    allImageFilesToBeReplaced.forEach(filePath => {
                        let parsedImagePath = path_1.default.parse(filePath);
                        // eg: "background.png"
                        let fileNameWithExtension = parsedImagePath.base;
                        let fileNameRegex = new RegExp(fileNameWithExtension, 'gi');
                        // eg: "background.webp"
                        let newImageName = parsedImagePath.name + '.webp';
                        // replace the image file name inside the dist's js, css and html files
                        fileContent = fileContent.replace(fileNameRegex, newImageName);
                        if (notLoggedYet) {
                            logBeautifiedConversionData(`\x1b[90mcreated: \x1b[32m${newImageName} - `, imageConversionData[i], options);
                            i++;
                        }
                    });
                    notLoggedYet = false;
                    // save the new file content
                    fs_1.default.writeFileSync(filePath, fileContent, 'utf-8');
                };
                // for all dist files (js, html and css)
                for (const file of allDistFiles) {
                    if (/\.(js|html|css)$/.test(file)) {
                        replacePaths(file);
                    }
                }
                /**
                 * aggregate the save image data
                 */
                let totalConversionData = {
                    before: 0,
                    after: 0,
                    percentage: 0
                };
                imageConversionData.forEach(e => {
                    totalConversionData.before += e.before;
                    totalConversionData.after += e.after;
                });
                logBeautifiedConversionData('\n\x1b[32mtotal savings: ', totalConversionData, options);
                console.log('\n');
            }
            catch (error) {
                console.log(`\x1b[31m\n${pluginName} - error:\n\x1b[0m`, error);
                console.log('\nYour current options: ', options);
            }
        }
    };
}
//# sourceMappingURL=plugin.js.map